
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "obex.h"
#include "vt.h"

#define  BLOCK_SIZE  512


obexsession *session = NULL;
char line[200];
char current_dir[1024] = "/";
char complete_path[1024] = "/";
char name[1024];


void change_dir(char *path) {
	if (path[0] == '/')
		strcpy(current_dir, path);
	else {
		if (current_dir[strlen(current_dir) - 1] != '/')
			strcat(current_dir, "/");
		strcat(current_dir, path);
	}
}


void get_name_from_path(char *path) {
	char *p;

	p = strchr(path, '/');
	if (p == NULL)
		strcpy(name, path);
	else {
		strcpy(name, path);
		while ((p = strchr(name, '/')) != NULL)
			memmove(name, p + 1, strlen(p + 1) + 1);
	}
	return;
}


void expand_path(char *path) {
	if (path[0] == '/')
		strcpy(complete_path, path);
	else {
		strcpy(complete_path, current_dir);
		if (complete_path[strlen(complete_path) - 1] != '/')
			strcat(complete_path, "/");
		strcat(complete_path, path);
	}
}


void list_dir(char *path) {
	int res;
	obexdirentry *dir_entry;

	expand_path(path);
	res = obex_readdir(session, complete_path);
	if (res != 0) {
		printf("Error reading dir '%s'\n", path);
		perror("list_dir");
		return;
	}
	printf("listing %s\n", complete_path);
	res = 0;
	while ((dir_entry = obex_nextentry(session)) != NULL) {
		if (dir_entry->isdir)
			printf("%30s\t<DIR>\n", dir_entry->name);
		else
			printf("%30s\t%7d\n", dir_entry->name, dir_entry->size);
		res++;
	}
	printf("%d files listed\n", res);
}


void get_file(char *path) {
	int size;
	int accumulator;
	unsigned char buffer[BLOCK_SIZE];
	FILE *f;

	expand_path(path);
	get_name_from_path(complete_path);
	printf("get_file:  path='%s'  name='%s'\n", complete_path, name);
	size = obex_get(session, complete_path, 0);
	if (size < 0) {
		printf("Error reading file '%s'\n", path);
		perror("get_file");
		return;
	}
	printf("getting %d bytes\n", size);
	f = fopen(name, "wb");
	if (f == NULL) {
		printf("Error opening local file for write\n");
		perror("get_file");
		return;
	}
	accumulator = 0;
	while ((size = obex_read(session, buffer, BLOCK_SIZE)) > 0) {
		fwrite(buffer, 1, size, f);
		accumulator += size;
		vt_backward(20);
		vt_clreol();
		printf("%d", accumulator);
		fflush(stdout);
	}
	printf(" byte(s)\n");
	obex_close(session);
	fclose(f);
}


void put_file(char *filename) {
	FILE *f;
	char mobile_path[1024];
	unsigned char buffer[BLOCK_SIZE];
	int size;
	int accumulator;

	f = fopen(filename, "rb");
	if (f == NULL) {
		printf("Error opening local file for read\n");
		perror("put_file");
		return;
	}
	strcpy(mobile_path, current_dir);
	if (current_dir[strlen(current_dir) - 1] != '/')
		strcat(mobile_path, "/");
	strcat(mobile_path, filename);
	printf("putting '%s' to '%s'\n", filename, mobile_path);
	if (obex_put(session, mobile_path) < 0) {
		printf("Error putting file\n");
		perror("put_file");
		return;
	}
	accumulator = 0;
	while ((size = fread(buffer, 1, BLOCK_SIZE, f)) > 0) {
		obex_write(session, buffer, size);
		accumulator += size;
		vt_backward(20);
		vt_clreol();
		printf("%d", accumulator);
		fflush(stdout);
	}
	printf(" byte(s)\n");
	fclose(f);
	obex_close(session);
}


int parse_command(char *command) {
	if (strncmp(command, "quit", 4) == 0)
		return 1;
	else if (strncmp(command, "ls", 2) == 0) {
		if (strlen(command) == 2)
			list_dir(current_dir);
		else
			list_dir(&(command[3]));
		return 0;
	}
	else if (strncmp(command, "cd ", 3) == 0) {
		change_dir(&(command[3]));
		return 0;
	}
	else if (strncmp(command, "get ", 4) == 0) {
		get_file(&(command[4]));
		return 0;
	}
	else if (strncmp(command, "put ", 4) == 0) {
		put_file(&(command[4]));
		return 0;
	}
	else if (strcmp(command, "pwd") == 0) {
		printf("Current directory: '%s'\n", current_dir);
	}
	else {
		printf("Command not found:\n");
		printf("\tquit      Exit siefsconsole\n");
		printf("\tls DIR    List directory\n");
		printf("\tcd DIR    Change to directory\n");
		printf("\tget FILE  Get the FILE from mobile phone\n");
		printf("\tput FILE  Put the FILE into the current directory at mobile phone\n");
		printf("\tpwd       Show current directory path\n");
	}
	return 0;
}

void show_copying() {
	printf("SieFSConsole (c) 2004 Avelino Herrera Morales\n\n");
	printf("Based on SieFS (http://chaos.allsiemens.com/siefs) by Chaos.\n");
	printf("GNU General Public License. See COPYING file for details.\n\n");
}

int main(int argc, char *argv[]) {
	char *p;

	show_copying();
	if (argc < 2) {
		fprintf(stderr, "Syntax: %s device\n", argv[0]);
		exit(-1);
	}
	session = obex_startup(argv[1], 19200);
	if (session == NULL) {
		printf("Error opening OBEX session at %s\n", argv[1]);
		perror("main");
	}
	printf("OBEX session opened at %s\n", argv[1]);
	while (1) {
		printf("$ ");
		/* scanf("%[^\n]", line); */
		fgets(line, 190, stdin);
		p = strchr(line, '\n');
		if (p != NULL)
			*p = '\0';
		if (parse_command(line))
			break;
	}
	obex_shutdown(session);
	return 0;
}
